
/*  @(#)mimep.c 1.3 95/07/07
 *
 *  Copyright (c) Mikael Cam.
 *                All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged.
 *
 *  No responsibility is taken for any errors inherent either
 *  to the comments or the code of this program, but if reported
 *  to me then an attempt will be made to fix them.
 */

#include "mimep.h"


/************************************************************************/

int main(int narg, char **tabarg)

/*
 * Main function.
 *
 *
 *
 */

{
  
  char *inputfile;
  char *outputfile;
  char outputstr[MAXARRAY];
  char boundarystr[MAXARRAY];
  char rulehstr[MAXARRAY];
  char printerstr[MAXARRAY];
  char comm_mm[MAXARRAY];
  char comm_latex[MAXARRAY];
  char comm_dvips[MAXARRAY];
  char comm_lpr[MAXARRAY];
  char file[MAXARRAY];
  char *lastdir;
  char *latexdir;


  
  argc = narg;
  argv = tabarg;
  toprinter = True;
  printername = NULL;

/* If the MIMEPTMPDIR environment variable is set with a valid
 * directory name, mimep uses this directory to create its
 * temporary files.
 */

  if(!(TMPDIR = getenv(MIMEPTMPDIR)))    
    {
      TMPDIR = (char *) malloc(sizeof(char) * (strlen(DEFTMPDIR) + 1));
      strcpy(TMPDIR, DEFTMPDIR);
    }



  Header() ;   /* mp */


/* Treats the X-resources and the options of the command line */

  treat_options(); 


/* Determines the input file and opens it */
  inputfile = inputfilename();
  if (!(fp_in = fopen(inputfile, "r"))) 
    {
      fprintf(stderr, ERR1);
      usage(args, comments);
    }
  

/* Idem with the output file + removes the suffix of the file */

  outputfile = removesuffix(outputfilename());

  if (!(strcmp(outputfile, NOOUTPUTFILE)))
    outputfile = tempnam(TMPDIR, TMPREFIX);
  
/* Opens the output file which will contain the LaTeX document */

    if (!(fp_out = fopen(outputfile, "w")))
      {
	fprintf(stderr, ERR3);
	exit(1);
      }


/* Initializes the MIMEPOUT environment variable with the output file name
 * to communicate it to its collaborators (printmim ...)
 */

  strcpy (outputstr, MIMEPOUT);
  strcat (outputstr, outputfile);
  putenv (outputstr);
  if(_TRACE) fprintf(stderr, "MIMEPOUT: %s\n", outputstr);


/* Sets the MIMEPBOUND environment variable to default, non or to
 * the name of a PostScript file, depending of the boundary the
 * user wants between the bodyparts
 */


  strcpy (boundarystr, MIMEPBOUND);

  if ((!boundary) || (!strcmp(boundary, DEFAULTB)))
    {
      strcat (boundarystr, DEFAULTB); /* For a simple horizontal line */

      strcpy (rulehstr, MIMEPRULEH); /* Sets the environment variable */
      strcat (rulehstr, RULEH);      /* which defines the height of   */
      putenv (rulehstr);             /* this line.                    */     
    }
  else
    strcat (boundarystr, boundary);
  putenv (boundarystr);

/* Sets the MIMEPPRINT environment variable to the name of the printer
 * the user wants mimep to print on. Interesting for shell scripts 
 * used by mimep.
 */

  if(printername)
    {
      strcpy(printerstr, MIMEPPRINT);
      strcat(printerstr, printername);
      putenv(printerstr);
    }


  maxcarnb (typesize);

  if(_TRACE) fprintf(stderr, "MIMEPBOUND: %s\n", boundarystr);

/* Copy the LaTeX prologue to the LaTeX file */

  latex_header();


/* Translates the RFC 822 headers to LaTeX and add it to the LaTeX
 * file, with a pretty presentation (you can modify...)
 */

  mk_headers();
  
/* Closes the LaTeX file */

  fclose(fp_out);

/* Builds the metamail's call command line */
  strcpy(comm_mm, METAMAIL);
  strcat(comm_mm, inputfilename());
  strcat(comm_mm, TONULL);
  if(_TRACE) fprintf(stderr, "Commande: %s\n", comm_mm); 
  
/* Calls metamail */

  system(comm_mm); 

/* Opens the LaTeX output file to add the end of the document */
 
    if (!(fp_out = fopen(outputfile, "a")))
      {
	fprintf(stderr, ERR3);
	exit(1);
      }
     
/* Adds the end of the document */

  latex_enddoc();
  
/* Flushes the output file's buffer then closes the file  */

  fflush (fp_out);
  fclose (fp_out);
  

/* Store the path of the current directory */ 

  if(!(lastdir = getcwd(NULL, MAXARRAY)))
    fprintf(stderr,"%s %s", ERR4, lastdir);

/* Chdir to the directory of the LaTeX file */

  if (latexdir = getdir(outputfile))
    if((chdir(latexdir)) == -1)
      fprintf(stderr, "%s %s", ERR5, latexdir);

/* Builds the LaTeX's call command line */

  strcpy (comm_latex, LATEX);
  strcat (comm_latex, outputfile);
  strcat (comm_latex, TONULL);

  if(_TRACE) fprintf(stderr, "comm_latex: %s\n", comm_latex);

/* Calls LaTeX on our LaTeX document */

  system(comm_latex); 
  

/* Returns to the original directory */
  if((lastdir) && (latexdir))
    if((chdir(lastdir)) == -1)
      fprintf(stderr, ERR6);

/* Builds the dvips' call command line then calls dvips */

  strcpy (comm_dvips, DVIPS);
  strcat (comm_dvips, outputfile);
  


  if (tofile)
    {
      strcat (comm_dvips, DVIPSO);
      strcat (comm_dvips, outputfile);
      strcat (comm_dvips, PSSUFFIX);
      system (comm_dvips); /* For an output to a file */
    }

  if (toprinter) /* For an output to a printer */
    {

      if (tofile)
	{
	  strcpy(comm_lpr, LPR); /* Uses the lpr command ... */

	  if (printername)
	    {
	      strcat(comm_lpr, LPRP);
	      strcat(comm_lpr, printername);
	    }
	  if (!burstpage) strcat(comm_lpr, LPRH);
	  strcat(comm_lpr, outputfile);
	  strcat(comm_lpr, PSSUFFIX);
	  system(comm_lpr);
	}
      else
	{
	  strcat(comm_dvips, DVIPSO);
	  strcat(comm_dvips, TOPRINTER);
	  if (!burstpage) strcat(comm_dvips, LPRH);
	  if (printername)
	    {
	      strcat(comm_dvips, LPRP);
	      strcat(comm_dvips, printername);
	    }
	  strcat(comm_dvips, ENDVIPSO);
	  system (comm_dvips); 
	}
  if(_TRACE) fprintf(stderr, "comm_lpr: %s\n", comm_lpr);
    }
  else
    if(tostdout)        /* For the standard output */
      {
	strcat (comm_dvips, DVIPSF);
	system (comm_dvips);
      }
 
  if (_TRACE) fprintf(stderr, "comm_dvips: %s\n", comm_dvips);

  clean(); /* Removes the temporary files */

}


/************************************************************************/

void latex_header(void)

/*
 * Writes the LaTeX prologue in the LaTeX file. 
 * (paper size, characters'size etc., beginning of the document ...)
 *
 *
 */


{

fprintf(fp_out, "\\documentclass [%s,%dpt] {article}\n", 
	(paper_size == P_A4) ? A4PAPER : USPAPER, typesize);
FP("\\usepackage[dvips]{graphics}");
FP("\\usepackage{moreverb}");
FP("\\usepackage[dvips]{color}");
FP("\\usepackage{float}");
fprintf(fp_out, "\\textwidth=%.2fpt\n", TEXTWIDTH);
fprintf(fp_out, "\\oddsidemargin=%dpt\n", ODDSIDEMARGIN);
fprintf(fp_out, "\\topmargin=%dpt\n", TOPMARGIN);
fprintf(fp_out, "\\headheight=%dpt\n", HEADHEIGHT);
fprintf(fp_out, "\\headsep=%dpt\n", HEADSEP);
fprintf(fp_out, "\\footskip=%dpt\n", FOOTSKIP);
fprintf(fp_out, "\\fboxsep=%dpt\n", FBOXSEP);
fprintf(fp_out, "\\textheight=%dpt\n",
	(paper_size == P_A4) ? A4TEXTHEIGHT : USTEXTHEIGHT);
FP("\\newlength{\\boxlength}");
fprintf(fp_out, "\\setlength{\\boxlength}{%.2fpt}\n", BOXLENGTH);
fprintf(fp_out, "\\addtolength{\\boxlength}{-%dpt}\n", (2 * FBOXSEP - 6));
fprintf(fp_out, "\\definecolor{linecolor}{gray}{%.2f}", BOXLINECOLOR);
fprintf(fp_out, "\\definecolor{bckgrdcolor}{gray}{%.2f}", BOXBCKGRDCOLOR);
FP("\\newcommand{\\titleh}[1]{%");
FP("\\begin{flushleft}%");
FP("\\fcolorbox{linecolor}{bckgrdcolor}{\\parbox{\\boxlength}{#1}}%");
FP("\\end{flushleft}}");
FP("\\newenvironment{ident}{\\begin{list}%");
FP("{}{\\rightmargin=0mm}\\item[]}%");
FP("{\\end{list}}");
FP("\\newenvironment{indentright}{\\begin{list}%");
FP("{}{\\rightmargin=\\leftmargin%");
FP("\\leftmargin=0mm}\\item[]}%");
FP("{\\end{list}}");
FP("\\newenvironment{nofill}{\\parindent=0mm\\obeylines}{\\par}");
if (!printpagenum)
  FP("\\pagestyle{empty}");
FP("\\begin{document}");

}

/************************************************************************/

void latex_enddoc(void)

/*
 * Writes the end of the LaTeX document in the LaTeX file.
 *
 *
 *
 */


{

FP("\\end{document}");

}

/************************************************************************/

char *inputfilename(void)

/*
 * Returns the name of the input file treated by the command.
 *
 *
 *
 */


{
  
  char *filename;
  char car;
  FILE *fp;

  if ((argc > 1) && (!isoption(argv[1]))) 
    {
      fromstdin = False;
      return (argv[1]);
    }
  else
    {

/* If mimep uses the standard input, dumps the informations into a temporary file,
 * then, returns the name of this file.
 */


    filename = tempnam(TMPDIR, TMPREFIX); 
    if (!(fp = fopen(filename, "w"))) fprintf(stderr, ERR1);
    while(fscanf(stdin, "%c", &car) != EOF)
      fprintf(fp, "%c", car); 
    fclose(fp);
    fromstdin = True;
    return (filename);
    }

}

/************************************************************************/

void mk_headers(void)

/*
 * Add a banner to the LaTeX document with the name of the user who prints the
 * message, the correspondants' addresses, the subject and the date of the message.
 *
 *
 */


{
     char *ch;
     char **to;
     char **cc;
     char titlefrom[MAXF];
     int  i;
     FILE *fp;
     struct stat buf;

     set_input(fp_in);

     readline() ; 

     if (end_of_file())
       {
	 fprintf(stderr, ERR2) ;
	 exit(1) ;
       }
     
/* parses the headers */

     parse_headers(0) ;  /* from mp */

     init_setup();       /* from mp */

/* Builds the banner box on top of the message with the RFC 822 headers,
 * then adds them to the LaTeX file.
 */

     FP("\\begin{flushleft}");

     fprintf(fp_out, "\\Large{%s \\textbf{%s}}\n",
	     TITLEH_TO, (owner) ? text2latex(owner) : UNKNOWN); 
     
     if (ch = get_from_hdr())
       {
	 if((MAXTFROM) && (strlen(ch) > MAXTFROM))
	   {
	     strncpy(titlefrom, text2latex(ch), MAXTFROM);
	     titlefrom[MAXTFROM] = EOS;
	   }
	 else
	   strcpy(titlefrom, text2latex(ch));
       }

     fprintf(fp_out, "\\titleh{\\large{\\begin{description}\\itemsep=%dpt\\item[\\textnormal{%s}] \\textbf{%s}%%\n",
	     ITEMSEP, TITLEH_FROM,  (ch) ? titlefrom : NONE);
	     
     fprintf(fp_out, "\\item[\\textnormal{%s}] \\textbf{%s}%%\n",
	     TITLEH_SUBJECT, (ch = get_subject_hdr()) ? text2latex(ch) : NONE);

     fprintf(fp_out, "\\item[\\textnormal{%s}] \\textbf{%s}\\end{description}}}\n",
	     TITLEH_DATE, (ch = get_date_hdr()) ? text2latex(ch) : NONE);

     fprintf(fp_out, "\\vspace{%dpt}", SEP1);
   
/* Adds extra headers depending on the user's configuration */

     if (istoprint(FROMHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n",
	       H_FROM, (ch = get_from_hdr()) ? text2latex(ch) : NONE);

     if (istoprint(TOHDR))
       {
	 to = get_to_hdr();
	 fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_TO,
		 (to[0]) ? text2latex(to[0]) : NONE);
	 i = 1;
	 while(to[i]) { fprintf(fp_out, "\\small{ %s}\\\\\n", text2latex(to[i])); i++; }
       }

     if (istoprint(SUBJECTHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_SUBJECT,
	       (ch = get_subject_hdr()) ? text2latex(ch) : NONE); 

     if (istoprint(DATEHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_DATE,
	       (ch = get_date_hdr()) ? text2latex(ch) : NONE); 

     if (istoprint(CCHDR))
       {
	 cc = get_cc_hdr();
	 fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_CC,
		 (cc[0]) ? text2latex(cc[0]) : NONE);
	 i = 1;
	 while(cc[i]) { fprintf(fp_out, "\\small{ %s}\\\\\n", text2latex(cc[i])); i++; }
       }
     if (istoprint(APP_FROMHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_APPFROM,
	       (ch = get_appfrom_hdr()) ? text2latex(ch) : NONE); 

     if (istoprint(APP_TOHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_APPTO,
	       (ch = get_appto_hdr()) ? text2latex(ch) : NONE); 

     if (istoprint(REPLYHDR))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_REPLY,
	       (ch = get_replyto_hdr()) ? text2latex(ch) : NONE); 
     
     if ((istoprint(NEWSGROUPSHDR)) || (istoprint(NEWSGROUPHDR)))
       fprintf(fp_out, "\\small{{\\bf %s} %s}\\\\\n", H_NEWSGROUP,
	       (ch = get_newsgroups_hdr()) ? text2latex(ch) : NONE); 

/* Adds a boundary. */
          
     if (boundary)
       {
	 if (strcmp(boundary, NOB))
	   {
	     FP("\\begin{center}");

	     if (!strcmp(boundary, DEFAULTB))
	       fprintf(fp_out, "\\rule{\\textwidth}{%spt}\n", RULEH);
	     else
	       if (!stat(boundary, &buf))
		 fprintf(fp_out,"\\includegraphics{%s}\n", boundary);
		 
	       
	     FP("\\end{center}");
	   }
       }
     
     fprintf(fp_out, "\\vspace{%dpt}", SEP2);
     FP("\\end{flushleft}");

}

/************************************************************************/

char *outputfilename(void)

/*
 * Returns the name of the output file.
 *
 *
 *
 */


{
  

  int nopt;
  
  if ((nopt = isopt("-o")) && (nopt + 1 < argc)) 
    return (argv[nopt + 1]);
  else return(NOOUTPUTFILE);
 /* Returns NOOUTPUTFILE if the user doesn't want any */
  
}

/************************************************************************/

bool istoprint(char *hdr)

/*
 * If the RFC 822 header referenced by hdr is to print, returns True,
 * else False.
 *
 *
 */


{
  if (((printable(hdr)) && ((!issubopt(isopt("-rmhdrs"), hdr)))) ||
    (issubopt(isopt("-addhdrs"), hdr)) || (isopt("-allheaders"))) return (True);
  return (False);
}

/************************************************************************/

char *removesuffix (char *src_filename)

/*
 * Removes the suffix of a file in a path.
 *
 *
 *
 */


{

  int i;
  static char *tgt_filename;
  int nbcar = 0;
  
  for (i = 0; i < strlen (src_filename); i++)
    {
      if (src_filename[i] == SLASH) nbcar = 0;
      if (src_filename[i] == POINT) 
	nbcar = i;
    }
  if (!nbcar) return (src_filename);

  tgt_filename = (char*) malloc (sizeof(char) * (nbcar + 1));
  
  strncpy (tgt_filename, src_filename, nbcar);
  tgt_filename[nbcar] = EOS;

  return (tgt_filename);
}

/************************************************************************/

void maxcarnb (int typeSize)

/*
 * Sets the MIMEPMAXCH environment variable to the maximum number
 * of characters in a line.
 * (for the cutlines command for instance ...)
 *
 */


{
  static char maxcarstr[16];
  
  strcpy (maxcarstr, MIMEPMAXCH);
  strcat (maxcarstr, nbmaxcar[typeSize - 10]);
  putenv (maxcarstr);
}

/************************************************************************/

char *text2latex(char *src_str)

/*
 * Converts a simple text line into LaTeX.
 * (useful when there are some special symbols in the text.)
 *
 *
 */


{
  char *tgt_str;
  int  i,
       k;
  bool trouve;
  char carcour;
  int  length;

  if(!src_str) return(NULL);
  tgt_str = (char *) malloc (sizeof(char) * 1024);
  
  tgt_str[0] = EOS;

  for(i = 0; i < strlen(src_str); i++)
    {
      k = 0;
      trouve = False;
      carcour = src_str[i];

		  while ((specar[k]) && (!trouve))
		    {
		      if (((char)*specar[k] == carcour) && (specar[k+1]))
			{
			  strcat(tgt_str, specar[k+1]); 
			  trouve = True;
			}
		      if(!trouve) k+=2;
		    }
      if(!trouve)
        {
	  length = strlen(tgt_str);
	  tgt_str[length] = carcour;
	  tgt_str[length + 1] = EOS;
	}
    }
  return(tgt_str);
}

/************************************************************************/

char *getdir(char *path)

/*
 * Returns the path to a file.
 *
 *
 *
 */


{
  static char *path_ret;
  char *pslash;
  path_ret = (char *) malloc (sizeof(char) * (strlen(path) + 1));
  strcpy(path_ret, path);
  
  if (pslash = rindex(path_ret, SLASH))
    {
      *pslash = EOS;
      return(path_ret);
    }
  return(NULL);
}

/************************************************************************/

void treat_options(void)

/*
 * Treats the X-resources and the options of the command line.
 *
 *
 *
 */



{
  int  optnum;
  char mailcapath[MAXARRAY];
  
  Option() ;   /* mp */
  
  get_appname(argc, argv) ;  /* mp */
  
  read_resources();  /* mp */

/* Verifies wether the command line is coherent (with the usage) or not ... */

  if (!iscoherent_com_line(argc, argv, args, 0)) usage(args, comments);
  

/* Determines the name of the printer on which it can print */

  printername = str_opt_val(O_PR_NAME);
  if (optnum = isopt("-printer"))
    printername = argv[optnum + 1];
  

/* Determines if the output is to a file */

  if (isopt("-o"))
    {
      tofile = True; 
      toprinter = False;
    }
  else
    tofile = False;

/* Do we have to print even if the -o option is set ? */

  if ((isopt("-fp")) || (isopt("-printer")))
    toprinter = True;
  
/* Is the output the standard output ? */

  if (isopt("-f"))
    {
      toprinter = False;
      tofile = False;
      tostdout = True;
    }
  else 
    tostdout = False;
  
/* Do we have to print the burst page ? */

  if (int_opt_val(O_BURSTPAGE))
    burstpage = True;
  else
    burstpage = False;

  if (isopt("-noburstpage"))
    burstpage = False;
  
/* Determines the paper size. */

  if(paper_size == P_NONE) paper_size = P_A4;

  if (isopt("-a4"))
    paper_size = P_A4;  
  else 
    if (isopt("-us")) 
    paper_size = P_US;

/* same with type size (characters) */

  if ((optnum = isopt("-typesize")) && (optnum + 1 < argc))
    typesize = atoi(argv[optnum + 1]);
    
  if (typesize < MINTSIZE) typesize = MINTSIZE;
  else
    if (typesize > MAXTSIZE) typesize = MAXTSIZE;

/* Do we have to print the page numbers ? */

  if (!int_opt_val(O_PAGIN))
    printpagenum = False;
  else
    printpagenum = True;

  if (isopt("-nopagenum"))
    printpagenum = False;

/* Sets the path of the mailcap file if the user specified one. */

  if (optnum = isopt("-mailcap"))
    {
      strcpy (mailcapath, MIMEPMAILCAP);
      strcat (mailcapath, argv[optnum + 1]);
      putenv (mailcapath);
    }



 if(_TRACE) 
   {  

     fprintf(stderr, "paperSize: %s\n", (paper_size == P_A4) ? "A4" : "US");
     fprintf(stderr, "boundary:  %s\n", (boundary) ? boundary : "");
     fprintf(stderr, "typeSize:  %d\n", typesize);
     fprintf(stderr, "paginate:  %s\n", (printpagenum) ? "True" : "False");
  
   }

}

/************************************************************************/

void clean(void)

/*
 * Removes the temporary files.
 *
 *
 *
 */


{
  char comm_rm[MAXARRAY];

  strcpy(comm_rm, RM);
  strcat(comm_rm, TMPDIR);
  strcat(comm_rm, SLASHSTR);
  strcat(comm_rm, TMPREFIX);
  strcat(comm_rm, STAR);
  system(comm_rm);

  strcpy(comm_rm, RM);
  strcat(comm_rm, TMPDIR);
  strcat(comm_rm, SLASHSTR);
  strcat(comm_rm, MMPREFIX);
  strcat(comm_rm, STAR);
  system(comm_rm);
  


}

/************************************************************************/







